package program;

import java.io.File;
import java.util.ArrayList;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.commentator.CommentAtor;
import program.settings.Container_Settings;
import program.settings.XhtmlSettings;
import program.utilities.IOUtils;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Contains all necessary information about a project which has to be parsed.
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class Container_Files {
	
	private static final I18n i18n = I18nFactory.getI18n(Container_Files.class, "bin.l10n.Messages", program.Globals.getLocale());
	private static final CommentAtor ca = CommentAtor.getInstance();

	public Container_Settings sc = new Container_Settings();
	
	public static class Container {

		private ArrayList<File> files = new ArrayList<File>(); // All used files
		
		File commonFile; // File containing the default settings
		File footerFile; // File containing the footer
		File menuFile; // Menu file containing the menu
		
		public File styleDir; // Folder containing the stylesheets and the reck
		private File targetDir;
		public File targetDir() {
			if (targetDir == null)
				Container_Files.getInstance().setTargetDirectory(Constants.Directories.target);
			return targetDir;
		}
		
		private File targetSDir; // Target style directory
		public File targetSDir() {
			targetDir();	// Set up target directory if necessary; targetSDir will be created too
			return targetSDir;
		}
		
		String pastedTextCode = new String(); // Code from the pasted text 
		String pastedText; // Pasted Text in the code window
		
		/** @return Template for the pages */
		public File reck() { 
			return new File(styleDir.getPath() + File.separatorChar +
					XhtmlSettings.getInstance().local.getReck()
					); 
		}
		
		/** @return Template for the image page */
		public File imageTemplate() { return new File(styleDir.getPath() + File.separatorChar + Constants.Files.imageTemplate); }
		
		public File cssSettings() { return new File(styleDir.getPath() + File.separatorChar + Constants.Files.cssSettings); }
		
	}

	public static class Has {
		/** Is there pasted text? */		boolean paste = false;
		/** Is the menu file defined? */ 	boolean menu = false;
		/** Is a style directory given? */	boolean style = false; 
		/** Is a target directory given? */	boolean target = false; 
		/** Is a target style dir given? */ boolean targetS = false;
		/** Is a footer file given? */		boolean footer = false; 
		/** Don't correct typos? */			boolean noTypo = false;
		/** Is a common file set? */ 		boolean common = false;
	}
	
	public String currentFilename;
	public Container cont = new Container();
	public Has has = new Has();
	
	private Container_Files() {	}
	private static Container_Files fc = new Container_Files();
	
	public static Container_Files getInstance() { return fc; }

	public void clear() {
		cont = new Container();
		has = new Has();
	}

	/**
	 * Add an menu file (containing the menu) and test whether it exists
	 * 
	 * @param filename
	 *            name of the file
	 * @return true, if file successfully loaded
	 */
	public boolean addMenuFile(String filename) {
		has.menu = false;
		cont.menuFile = new File(filename);
		
		if (cont.menuFile.exists() && cont.menuFile != null) {
			has.menu = true;
			return true;
		}
		ca.ol(i18n.tr("Error") + ": " + i18n.tr("Menu file") + ' ' 
				+ i18n.tr("not found or file empty") + ": " + filename, CommentAtor.ERRORS);
		return false;
	}

	/**
	 * Add a file that has to be converted to the list.
	 * 
	 * @param filename
	 *            name of the file
	 * @return true, if success
	 */
	public boolean addFile(String filename) {
		File f = null;
		if (!fulfilsTextFileQualifications(filename))
			return false;
		filename = IOUtils.getShortPath(new File(filename), Constants.Directories.workingDir2);

		/*
		 * Try to open file, only return true if it has been done
		 * successfully.
		 * 
		 * File is first created seperately, not in the list.
		 */
		f = new File(filename);
		
		if (f != null && f.exists()) {
			/*
			 * Add the file to the list, if it does not exist already
			 */
			if (!cont.files.contains(f))
				cont.files.add(f);
			return true;
		}
		
		ca.ol(i18n.tr("Error") + ": " + i18n.tr("File") + ' ' + i18n.tr("not found or file empty") 
				+ ": " + filename, CommentAtor.ERRORS);
		return false;

	}

	public File getFile(int position) {
		return cont.files.get(position);
	}
	
	/** @return The number of files which are «saved» in the Array */
	public short getFilesNumber() { return (short) cont.files.size(); }

	public File getLastFile() {
		return cont.files.get(cont.files.size() - 1);
	}

	
	/**
	 * @return The absolute path of the style directory if available 
	 * or an emtpy string otherwise
	 */
	public String getStyleDirAbsolutePath() {
		return cont.styleDir == null ? ""
				: cont.styleDir.getAbsolutePath();
	}
	
	/**
	 * Sets the pasted text in the code window
	 * @param s
	 */
	public void setPastedText(String s) {
		if (s == null || s.equals( "" )) {
			has.paste = false;
			cont.pastedText = "";
		} else {
			has.paste = true;
			cont.pastedText = s;
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			xhs.local.content = new StringBuffer(s);
			cont.pastedTextCode = XHTML.wiki2xhtml().toString();
		}
	}

	public boolean setCommonFile(String filename) {
		boolean ok = fulfilsTextFileQualifications(filename);
		if (ok) {
			filename = IOUtils.getShortPath(new File(filename), Constants.Directories.workingDir2);
			cont.commonFile = new File(filename);
			has.common = true;
		}
		return ok;
	}

	public boolean setFooterFile(String filename) {
		boolean ok = fulfilsTextFileQualifications(filename);
		if (ok) {
			filename = IOUtils.getShortPath(new File(filename), Constants.Directories.workingDir2);
			cont.footerFile = new File(filename);
			has.footer = true;
		} else {
			has.footer = false;
			cont.footerFile = null;
		}
		return ok;
	}

	public boolean setMenuFile(String filename) {
		boolean ok = fulfilsTextFileQualifications(filename);
		if (ok) {
			filename = IOUtils.getShortPath(new File(filename), Constants.Directories.workingDir2);
			cont.menuFile = new File(filename);
			has.menu = true;
		} else {
			has.menu = false;
		}
		return ok;
	}

	public void setNoTypo(boolean b) {
		has.noTypo = b;
	}

	public boolean setStyleDirectory(String name) {
		boolean ok = fulfilsDirectoryQualifications(name);
		if (!ok) {
			name = Constants.Directories.style + File.separatorChar + name;
			ok = fulfilsDirectoryQualifications(name);
		}
		if (ok) {
			cont.styleDir = new File(name);
			has.style = true;
		}
		// Update target style directory! 
		if (has.target) {
			setTargetDirectory(cont.targetDir.getPath());
		}
		return ok;
	}

	public boolean setTargetDirectory(String name) {
		boolean ok = !IOUtils.isFile(name);
		if (!ok) {
			StringBuffer fn = new StringBuffer(name);
			int start;
			while ((start = fn.indexOf(File.separator)) >= 0 && !ok) {
				fn.delete(start, fn.length());
				if (!IOUtils.isFile(fn.toString()))
					ok = true;
			}
		}
		if (ok) {
			cont.targetDir = new File(name);
			has.target = true;
			
			if (has.style) {
				cont.targetSDir = new File(cont.targetDir.getPath() + File.separatorChar + Constants.Directories.style);
				has.targetS = true;
			}
		}
		return ok;
	}

	public void setOnlyCode(boolean b) {
		sc.onlyCode = b;
	}

	public void setStdOut(boolean b) {
		sc.stdOut = b;
	}

	public void setNoLinebreaks(boolean b) {
		sc.removeLineBreaks = b;
	}
	
	public Args getFileList() {
		Args a = new Args();
		for (int i = 0; i < cont.files.size(); i++)
			a.add(cont.files.get(i).getPath());
		return a;
	}

	public File[] getFileArray() {
		File[] fa = new File[cont.files.size()];
		for (short i = 0; i < cont.files.size(); i++) {
			fa[i] = cont.files.get(i);
		}
		return fa;
	}

	/**
	 * 
	 * @return String with all arguments except the files to convert, the
	 *         --stdout, the ones I forgot and the --help/verbose/... for
	 *         cmdline-Output.
	 */
	public Args getAllArguments() {
		Args a = new Args();

//		a.add("--lang=" + program.Globals.getLocale());
		
		if (has.menu) {
			a.add("-m ");
			a.add(cont.menuFile.getPath());
		}
		
		if (has.style) {
			a.add("-s ");
			a.add(cont.styleDir.getPath());
		}
		
		if (has.common) {
			a.add("-c ");
			a.add(cont.commonFile.getPath());
		}
		
		if (has.footer) {
			a.add("-f ");
			a.add(cont.footerFile.getPath());
		}
		
		if (sc.onlyCode) {
			a.add("--only-code");
		}
		
		if (has.noTypo) {
			a.add("--remove-linebreaks");
		}
		
		if (has.target) {
			a.add("--td ");
			a.add(cont.targetDir.getPath());	
		}	

		return a;
	}

	private boolean fulfilsTextFileQualifications(String filename) {
		boolean ok = true;

		File f = new File(filename);
		if (!IOUtils.fileExists(filename)) {
			ca.ol("File does not exist: " + filename, CommentAtor.V_MSG);
			ok = false;
		} else if (!IOUtils.isFile(filename)) {
			ca.ol("Is not a file (directory!): " + filename, CommentAtor.V_MSG);
			ok = false;
		} else if (IOUtils.binaryCheck(f)) {
			ca.ol("Is not a text file: " + filename, CommentAtor.V_MSG);
			ok = false;
		}

		return ok;
	}

	private boolean fulfilsDirectoryQualifications(String name) {
		boolean ok = true;

		if (!IOUtils.fileExists(name)) {
			ca.ol("Directory does not exist: " + name, CommentAtor.V_MSG);
			ok = false;
		} else if (!IOUtils.isDirectory(name)) {
			ca.ol("Is not a directory (file!): " + name, CommentAtor.V_MSG);
			ok = false;
		}

		return ok;
	}
	
	public void removeFooter() {
		has.footer = false;
		cont.footerFile = null;
	}
	public void removeMenu() {
		has.menu = false;
		cont.menuFile = null;
	}
	public void removeTargetDir() {
		has.target = false;
		cont.targetDir = null;
	}
	public void removeCommon() {
		has.common = false;
		cont.commonFile = null;
	}

}
