package program;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import program.typo.Formattings;
import program.utilities.IORead_Stats;
import program.utilities.StringTools;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Creates the html code for the menu.
 * 
 * @author 	Simon A. Eugster
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * ,		hb9eia
 * 
 * TODO 3 Untermenu nur schreiben, wenn aktiv
 */
public class WikiMenu {
	
	/** Search for a name in the entries */
	final static int NAME = 1 << 0;
	/** Search for a link in the entries */
	final static int LINK = 1 << 1;
	
	/** Contains the whole menu */
	private ArrayList<MenuEntry> menu = new ArrayList<MenuEntry>();

	/** deactivates links if open */ private static boolean link_deactivate = false;
	/** uses strong instead of a for open links */ private static boolean link_strong = false;
	
	public WikiMenu() {}
	
	/** TODO 4 use in style.config */
	public final int MAX_DEPTH = 3; 
	
	/** Reads the menu from a menu file */
	public void readNewMenu(File file) throws IOException {
		StringBuffer in = IORead_Stats.readSBuffer(file);
		
		BufferedReader b = new BufferedReader(new StringReader(in.toString()));
		ArrayList<String> al;
		String line;
		
		try {
			
			MenuEntry me = new MenuEntry();
			/** Stores the last ID of the parent from each level */
			int[] parentItem = new int[MAX_DEPTH];
			int i = 0;
			
			/** -1 means no parent item availible. */ 
			Arrays.fill(parentItem, -1);

			for (line = b.readLine(); line != null; line = b.readLine()) {
				/*
				 * Ignore comments (leading #). Parse the line. 
				 */
				if (line.length() > 0 && line.charAt(0) != '#') { 
					me = new MenuEntry();
					al = getCsvMenu(line);
					me.readLine(al);
					
					/* Set the parent item ID and clear the IDs of sub-parents */
					if (me.level > 0)
						parentItem[me.level - 1] = i;
					for (int j = me.level; j < MAX_DEPTH; j++)
						parentItem[j] = -1;
					
					/* Set the parent item */
					if (me.level >= 2 && parentItem[me.level - 2] >= 0) {
						me.parent = menu.get(parentItem[me.level - 2]);
					}
					
					menu.add(me);
					
	
					if (me.level >= 2 && parentItem[me.level - 2] >= 0) {
						menu.get(menu.size() - 1).parent.children.add(menu.get(menu.size() - 1));
					}
					
					i += 1;
					
				} else if (line.length() > 2 && line.startsWith("#!")) {
					/*
					 * Read settings
					 */
					if (line.contains(Constants.MenuTags.linkDeactivate))
						link_deactivate = true;
					if (line.contains(Constants.MenuTags.linkDeactivateNot))
						link_deactivate = false;
					if (line.contains(Constants.MenuTags.linkStrong))
						link_strong  = true;
					if (line.contains(Constants.MenuTags.linkStrongNot))
						link_strong = false;
				}
			}
			
		} catch (IOException e) { e.printStackTrace(); }
		
		
	}
	
	/**
	 * @return The generated menu in XHTML
	 */
	public StringBuffer getMenu() {
		return WikiLists.makeList(getMenuAsList());
	}
	
	/**
	 * @return The generated menu, as a Wiki-List
	 */
	public StringBuffer getMenuAsList() {
		StringBuffer s = new StringBuffer();
		boolean openChild;
		
		for (MenuEntry me : menu) {
			
			/*
			 * Check for open children in the group
			 */
			openChild = false;
			for (MenuEntry mm : getGroup(me))
				for (MenuEntry m : mm.children)
					if (m.isOpen()) {
						openChild = true;
						break;
					}
			
			/*
			 * Append the line
			 */
			s.append(me.getLine(
					checkOpen(getGroup(me)),
					(me.parent != null ? me.parent.isOpen() : false),
					openChild, link_deactivate, link_strong
					) 
					+ "\n");
		}
		
		return parseLineArgumentsNolist(s);
	}

	/**
	 * Opens an item. First closes all other items.
	 * @param text The item to open
	 * @return false, if the item couln't be found
	 */
	public boolean openItem(String text, int what) {
		Vector<Short> v;
		
		/*
		 * Find out whether the user wants to search for a link 
		 * or the text entry 
		 */
		if ((what & NAME) > 0)
			v = findByName(text);
		else
			v = findByLink(text);
		
		/*
		 * Close all entries
		 */
		closeAll();
		
		/*
		 * Open all matching entries
		 */
		if (v.size() > 0) {
			for (short s = 0; s < v.size(); s++)
				menu.get(v.get(s)).open();
			return true;
		}
		
		return false;
	}

	private void closeAll() {
		for (MenuEntry me : menu) {
			me.close();
		}
	}

	/**
	 * @param me
	 * @return The group of the MenuEntry, means this and other entries of the 
	 * same level which are not separated by an entry of a lower level 
	 */
	private Vector<MenuEntry> getGroup(MenuEntry me) {
		Vector<MenuEntry> v = new Vector<MenuEntry>();
		
		if (me == null) return v;
		
		Vector<Short> v2 = findByName(me.name);
		
		if (v2.size() > 0) {
			/**
			 * Add the MEs of the same group (inclusive «me» itself).
			 * They have to have the same level and may not have an item 
			 * with a lower level separating two entries. 
			 */
			for (short s = v2.get(0); s >= 0; s--) {
				if (menu.get(s).level == me.level)
					v.add(menu.get(s));
				else if (menu.get(s).level < me.level)
					break;
			}
			
			/**
			 * Add the remaining MEs
			 */
			for (int i = v2.get(0) + 1; i < menu.size(); i++) {
				if (menu.get(i).level == me.level)
					v.add(menu.get(i));
				else if (menu.get(i).level < me.level)
					break;
			}
		}
		
		return v;
	}
	
	/**
	 * @param v
	 * @return true, if any of the entries is open
	 */
	private boolean checkOpen(Vector<MenuEntry> v) {
		for (MenuEntry me : v)
			if (me.isOpen())
				return true;
		return false;
	}
	
	/**
	 * @return An ArrayList containing the input values separated by a comma
	 */
	private ArrayList<String> getCsvMenu(String s) {
		Pattern p = Pattern.compile("(?<!\\\\),");
		Matcher m = p.matcher(s);
		
		ArrayList<String> al = new ArrayList<String>();
		
		if (m.find()) {
			int end, start = 0;
			do {
				end = m.start();
				al.add(s.substring(start, end).replaceAll("\\\\,", ","));
				start = m.end();
			} while (m.find());
			if (end < s.length())
				al.add(s.substring(start , s.length() ).replaceAll("\\\\,", ","));
			
			
		} else
			if (s.length() > 0)
				al.add(s);
		
		return al;
	}
	
	private Vector<Short> findByName(String name) {
		Vector<Short> v = new Vector<Short>();
		
		for (short i = 0; i < menu.size(); i++)
			if (menu.get(i).name.equals(name))
				v.add(i);
		return v;
	}
	
	/**
	 * @param link
	 * @return
	 */
	private Vector<Short> findByLink(String link) {
		Vector<Short> v = new Vector<Short>();
		
		for (short i = 0; i < menu.size(); i++)
			if (menu.get(i).link.equals(link))
				v.add(i);
		return v;
	}
	
	private StringBuffer parseLineArgumentsNolist(StringBuffer in) {
		BufferedReader b = new BufferedReader(new StringReader(in.toString()));
		StringBuffer out = new StringBuffer();
		String line;
		
		String arguments;
		int pos;
		
		Pattern p = Resources.Regex.listGroupArguments;
		
		try {
			Matcher m;
			while ((line = b.readLine()) != null) {
				if (!line.startsWith("*")) {
					arguments = "";
					m = p.matcher(line);
					if (m.find()) {
						line = line.substring(m.end(), line.length());
					}
					if ((pos = line.indexOf('|')) >= 0) {
						arguments = line.substring(0, pos);
						line = line.substring(pos + 1, line.length());
					}
					out.append((arguments.length() > 0 ? "<h3 " + StringTools.removeSpacesBE(arguments) + ">": "<h3>")
							+ StringTools.removeSpacesBE(line)
							+ "</h3>\n");
				}
				else
					out.append(line + '\n');
				
			}
		} catch (IOException e) {
			e.printStackTrace();
			return in;
		}
		
		return out;
		
	}
	
	private class MenuEntry {
		String name = "";
		String link = "";
		String args = "";

		byte level = 0;
		boolean open = false;
		
		MenuEntry parent = null;
		ArrayList<MenuEntry> children = new ArrayList<MenuEntry>();
		
		public MenuEntry() { }
		
		public void reset() {
			level = 0;
			name = "";
			link = "";
			args = "";
			open = false;
		}
		
		/** marks the entry as open */
		public void open() { open = true; }
		
		/** marks the entry as closed */
		public void close() { open = false; }
		
		/** @returns true, if the entry is marked as open */
		public boolean isOpen() { return open; }
		
		/** @return true, if any child of any level is open. */
		public boolean openChild() { 
			for (MenuEntry me : children) {
				if (me.open || me.openChild())
					return true;
			}
			
			return false; 
		}
		
		/** @return true, if the (direct) parent is open. */
		public boolean openParent() { 
			if (parent != null)
				return parent.open;
			
			return false;
		}
		
		/** @return true, if the entry has a child. */
		public boolean hasChild() { return (children.size() > 0); }
		
		
		/**
		 * Reads the settings from a line 
		 * @param al
		 */
		public void readLine(ArrayList<String> al) {
			try {
				name = al.get(0);
				link = StringTools.removeSpacesBE(al.get(1));

				level = 0;
				while (name.length() > 0 && name.charAt(0) == '*') {
					level++;
					name = name.substring(1, name.length());
				}
				if (level > MAX_DEPTH)
					level = MAX_DEPTH;
				
				name = StringTools.removeSpacesBE(name);
				
				args = al.get(2);
				
			} catch (IndexOutOfBoundsException e) {}
			
		}
		
		/**
		 * @return The line for the Menu file in Wikitext
		 */
		public StringBuffer getLine(boolean openGroup, boolean openParent, boolean openChild, boolean link_deactivate, boolean link_strong) {
			StringBuffer line = new StringBuffer();

			if (name.startsWith("----"))
				line.append("<hr />");
			else {
				if (level > 0) {
					for (int i = 0; i < level; i++)
						line.append('*');
					line.append(' ');
				}
				
				/* Append the «ul» arguments */
				line.append("((class=\"" + getFirstArgs(openGroup, openParent, openChild, level) + "\")) ");
				
				/* Append the «li» arguments */
				line.append(getStyleArgs(args) + " | ");
				
				if (link.length() > 0)
					line.append(
							(link_strong && open ? "<strong>" : "") + 
							(link_deactivate && open ? "" : 
								"<a href=\"" +
									link + 
								"\">") + 
							formatEntry(name) + 
							(link_deactivate && open ? "" : "</a>") +
							(link_strong && open ? "</strong>" : "")
					);
				else
					line.append(
							(link_strong && open ? "<strong>" : "") +
							name +
							(link_strong && open ? "</strong>" : ""));
			}
			
			return line;
		}
		
		private String getFirstArgs(boolean open, boolean openParent, boolean openChild, int level) {
			String s = new String();
			
			s += "level" + level + ' ';
			if (open)
				s += "open ";
			if (openChild)
				s += "openchild ";
			if (openParent)
				s += "openparent ";
			
			return StringTools.removeSpacesBE(s);
		}
		
		private String getStyleArgs(String args) {
			String s = new String();

			if (isOpen())
				s += "open ";
			if (openChild())
				s += "openchild ";
			if (openParent())
				s += "openparent ";
			if (hasChild())
				s += "hasChild ";
			
			s = StringTools.removeSpacesBE(s + "level" + level);
			
			int i = args.indexOf("class=\"");
			if (i >= 0)
				args = args.substring(0, i + "class=\"".length()) + s 
						+ args.substring(i + "class=\"".length(), args.length());
			else
				args += " class=\"" + s + "\"";
			
			return args;
		}
		
		/**
		 * @return The entry with bold/italic type or a horizontal line
		 */
		private StringBuffer formatEntry(String in) {
			if ("----".equals( in ))
				return new StringBuffer("<hr />");
			else {
				StringBuffer out;
				out = Formattings.makeBoldType(new StringBuffer(in), false);
				out = Formattings.makeItalicType(out, false);
				return out;
			}
		}
		
	}

}
