package program.settings;

import java.io.File;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import program.Constants;
import program.Container_Files;
import program.Container_Resources;
import program.GenerateID;
import program.Resources;
import program.Template;
import program.argumentHandler.ArgumentItem;
import program.argumentHandler.ArgumentReader;
import program.commentator.CommentAtor;
import program.utilities.IOUtils;
import program.utilities.StringTools;

/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Reads and handles image settings
 * 
 * @since wiki2xhtml 3.3, 2&#x2a2f faster than the old one
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * 
 */
public class ImageSettings {

	private static final CommentAtor ca = CommentAtor.getInstance();
	
	/** Settings for current page */
	public PageSpecific pagespecific = new PageSpecific();
	
	/** Settings for one image */
	public Specific specific = new Specific();
	
	private ImageSettings() { }
	private static ImageSettings is = new ImageSettings();
	/** Singleton */
	public static ImageSettings getInstance() { return is; }
	
	public Template tDesc, tBlank, pDesc, pBlank;
	
	
	/**
	 * Replaces characters like "&amp;" and " " by "&amp;amp;" and "%20"
	 * @param s
	 * @return W3C conform URI String
	 */
	public static String htmlURI(String s) {
		for (String[] es : Resources.entities)
			s = s.replaceAll(es[1], es[0]);
		return s;
	}
	
	/**
	 * Clears page specific settings
	 */
	public void nextPage(String pagename) {
		pagespecific = new PageSpecific();
		pagespecific.pagename = pagename;
	}
	
	/**
	 * Clears the settings for the specific item
	 */
	public void nextItem() { specific = new Specific(); }
	
	/**
	 * Contains the settings for images on one page.
	 * Overrides {@link program.settings.ImageSettings.All}
	 */
	public static class PageSpecific {
		
		/** Page specific: Page&#x2019;s name */
		private String pagename = new String();
		
		/** Gallery counter for current page */
		public int counterGalleries = 0;

		
		public boolean useConsistentGallery() {
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			return xhs.local.useConsistentGallery != null ? xhs.local.useConsistentGallery : xhs.global.useConsistentGallery;
		}
		
		/** Returns the gallery ID in the form of gallery001 */
		public String getIdGallery() {
			StringBuffer sb = new StringBuffer();
			for (byte i = 0; i < 2 - Math.round(Math.log10(counterGalleries)); i++)
				sb.append("0");
			sb.append(counterGalleries);
			return "gallery" + sb;
		}
		
		
		/** @return The page specific image page directory */
		public String getImagepagesDir() {
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			String dir = xhs.local.dirImagepages.length() > 0 
					? xhs.local.dirImagepages 
					: xhs.global.dirImagepages;
			
			dir.replaceAll("\\/\\/+", "/");
			
			return dir.endsWith("/") ? dir : dir + "/"; 
		}
		
		public String getBacklinkDir() {
			String dirImagepages = getImagepagesDir();
			
			int n = StringTools.countString(dirImagepages, "/");
			
			if (dirImagepages.startsWith("./")) n--;
			if (!dirImagepages.endsWith("/")) n++;
			
			StringBuffer backlink = new StringBuffer();
			
			for (int i = 0; i < n; i++) {
				backlink.append("../");
			}
			
			return backlink.toString();
		}
	}
	
	/**
	 * Contains the settings for one image.
	 * Overrides {@link program.settings.ImageSettings.All}.
	 */
	public static class Specific {
		
		/** Image description */
		public StringBuffer desc = new StringBuffer();
		/** Thumbnail width */
		private String widthThumb = new String();
		/** Image width on the image page */
		private String widthPage = new String();
		/** Image caption */
		public String caption = new String();
		/** Long image description */
		public String longdesc = new String();
		/** Image path */
		public String pathImage = new String();
		/** Thumbnail position */
		public String position = new String();
		/** Text (gallery entries */
		public String text = new String();
		/** Thumbnail source */
		private String thumbSrc = new String();
		

		
		/** Link to previous image */
		public String prevLink = new String();
		/** Link to next image */
		public String nextLink = new String();
		
		/** Image ID */
		public String getID() { return GenerateID.getHexMD5id(pathImage, ""); }
		
		
		/** Direct link without image page? */
		public boolean direct = false;
		/** Thumbnail? */
		public boolean isThumbDesired = false;
		/** Gallery */
		public boolean isGallery = false;
		/** Small image? (No with has to be set for the image page) */
		public boolean isSmall = false;
		
		/** Has the image page been created? */
		public boolean isImagepageCreated = false;
		
		/** Gallery counter */
		public byte galleryCounter = 0;
		
		

		
		
		/**
		 * Reads arguments from an image tag ([[image:args]]).
		 * @param args Arguments (without image: and square brackets)
		 */
		public void readArguments(String args, boolean textOnly) {
			
			if (textOnly) {
				text = args;
				return;
			}
			
			ArrayList<ArgumentItem> a = ArgumentReader.getArguments(args);
			
			if (a.size() > 0) {
				pathImage = a.get(0).fullArg;
				a.remove(0);
			}
			
			Pattern pWidth = Pattern.compile("^\\d+(?:px|%)$");
			Matcher mWidth;
			Pattern pPWidth = Pattern.compile("^pw(?:idth)?=(\\d+(?:px|%))");
			Matcher mPWidth;
			
			for (ArgumentItem ai : a) {
				
				/* Display thumbnail? */
				if (Constants.Images.thumb.equals(ai.fullArg)) {
					isThumbDesired = true;
					continue;
				}
				
				/* Position */
				if (Constants.Position.left.equals(ai.fullArg)) {
					position = ai.fullArg;
					continue;
				}
				if (Constants.Position.right.equals(ai.fullArg)) {
					position = ai.fullArg;
					continue;
				}
				if (Constants.Position.center.equals(ai.fullArg)) {
					position = ai.fullArg;
					continue;
				}
				
				/* Direct? */
				if (Constants.Images.direct.equals(ai.fullArg)) {
					direct = true;
					continue;
				}
				
				if (Constants.Images.small.equals(ai.fullArg)) {
					isSmall = true;
					continue;
				}
				
				
				/* Width */
				mWidth = pWidth.matcher(ai.fullArg);
				if (mWidth.find()) {
					widthThumb = mWidth.group();
					continue;
				}
				
				/* Thumbnail source */
				if (ai.fullArg.startsWith(Constants.Images.pathThumb)) {
					thumbSrc = ai.fullArg.substring(Constants.Images.pathThumb.length(), ai.fullArg.length());
					continue;
				}
				
				/* Caption */
				if (ai.fullArg.startsWith(Constants.Images.captionShort)) {
					caption = ai.fullArg.substring(2, ai.fullArg.length());
					continue;
				}
				if (ai.fullArg.startsWith(Constants.Images.caption)) {
					caption = ai.fullArg.substring(8, ai.fullArg.length());
					continue;
				}
				
				/* Long description */
				if (ai.fullArg.startsWith(Constants.Images.longDesc)) {
					longdesc = ai.fullArg.substring(3, ai.fullArg.length());
					continue;
				}
				
				/* Image page width */
				mPWidth = pPWidth.matcher(ai.fullArg);
				if (mPWidth.find()) {
					widthPage = mPWidth.group(1);
					continue;
				}
				
				/* Description */
				if (desc.length() > 0) {
					desc.append('|');
				}
				desc.append(ai.fullArg);
			}
			
			// Check for non-ASCII characters
			Matcher m = Pattern.compile(Resources.pNotAscii).matcher(pathImage);
			if (m.find()) {
				ca.o("Image contains non-ASCII characters! They should be removed.", CommentAtor.ERRORS);
			}
		}

		
		/** Returns the specific width for the image on the image page */
		public String getWidthPage() {
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			return widthPage.length() > 0 ? widthPage : 
				(xhs.local.widthImgImagepages.length() > 0 ? xhs.local.widthImgImagepages : xhs.global.widthImgImagepages); 
		}
		
		/** @return The attribute for the width of the image on the image page */
		public String getWidthPageAttrib() { return getWidthPage().length() > 0 ? "width=\"" + getWidthPage() + "\"" : ""; }
		
		/** Returns the specific thumbnail width */
		public String getWidthThumb() {
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			if (isThumbDesired) {
				return widthThumb.length() > 0 ? widthThumb : 
					(xhs.local.widthThumbs.length() > 0 ? xhs.local.widthThumbs : xhs.global.widthThumbs);
			} else {
				return widthThumb.length() > 0 ? widthThumb : "";
			}
		}

		/** @return The attribute for the thumbnail width */
		public String getWidthThumbAttrib() { return getWidthThumb().length() > 0 ? "width=\"" + getWidthThumb() + "\"" : ""; }
		
		
		/**
		 * Gets the filename of the image.
		 * @return Image filename without the path
		 */
		public String getImageFilenameHtml() {
			return htmlURI(pathImage.substring(pathImage.lastIndexOf('/') + 1));
		}

		/** @return Image path with escaped spaces and &amp;&#x2019;s */
		public String getImagePathHtml() {
			return htmlURI(getImagePath());
		}

		/**
		 * Builds the filename and path for the image page.
		 * @return Image page path
		 */
		public String getImagepagePath(boolean filesystemPath, boolean filenameOnly) {
			if (pathImage.length() == 0) {
				ca.ol("No image path available! Unable to getImagepagePath().", CommentAtor.ERRORS);
				return null;
			}
			StringBuffer name = new StringBuffer();
			
			if (!filenameOnly) {
				if (filesystemPath) {
					name.append(Container_Files.getInstance().cont.targetDir().getAbsolutePath() + File.separatorChar);
				}
				name.append(is.pagespecific.getImagepagesDir());
				
				if (filesystemPath) {
					// Path for the file is required, likely to create it, so create the sub-directory if necessary
					File f = new File(name.toString());
					if (!f.exists())
						f.mkdirs();
				}
			}
			
			name.append(is.pagespecific.pagename.replace(".", "-"));
			name.append("_to_");
			name.append(pathImage.replace(".", "-").replace("/", "+").replace(" ", "_"));
			name.append(".html");
			
			return name.toString();
		}
		

		/** @return The link back to the calling image or thumbnail */
		public String getBacklinkPage() {
			ImageSettings is = ImageSettings.getInstance();
			return is.pagespecific.getBacklinkDir() + is.pagespecific.pagename + "#" + getID();
		}
		
		
		/** @return The appropriate template */
		public Template getTemplate() {
			if (isGallery) {
				if (text.length() > 0) {		// Text only
					return new Template(
								new File(Container_Files.getInstance().cont.styleDir.getAbsolutePath() + File.separatorChar + "galleryText.html"),  
								Container_Resources.readResource(Container_Resources.sgalleryText)
							);
				} else {
					if (desc.length() > 0) {	// Thumbnail with description
						return new Template(
									new File(Container_Files.getInstance().cont.styleDir.getAbsolutePath() + File.separatorChar + "galleryImage.html"),  
									Container_Resources.readResource(Container_Resources.sgalleryImage)
								);
					} else {					// Thumbnail without description
						return new Template(
									new File(Container_Files.getInstance().cont.styleDir.getAbsolutePath() + File.separatorChar + "galleryImageNodesc.html"),  
									Container_Resources.readResource(Container_Resources.sgalleryImageNodesc)
								);
					}
					
				}
			}
			if (isThumbDesired) {
				if (desc.length() > 0)		// Thumbnail with description
					return new Template(
								new File(Container_Files.getInstance().getStyleDirAbsolutePath() + File.separatorChar + "thumbPictureT.html"), 
								Container_Resources.readResource(Container_Resources.sthumbPictureT)
							);
				else						// Thumbnail without description
					return new Template(
								new File(Container_Files.getInstance().getStyleDirAbsolutePath() + File.separatorChar + "thumbPicture.html"), 
								Container_Resources.readResource(Container_Resources.sthumbPicture)
							);
			} else {
				if (desc.length() > 0)		// Image with description
					return new Template(
								new File(Container_Files.getInstance().getStyleDirAbsolutePath() + File.separatorChar + "pictureT.html"), 
								Container_Resources.readResource(Container_Resources.spictureT)
							);
				else						// Image without description
					return new Template(
								new File(Container_Files.getInstance().getStyleDirAbsolutePath() + File.separatorChar + "picture.html"), 
								Container_Resources.readResource(Container_Resources.spicture)
							);
			}
		}
		
		/**
		 * 
		 * @return The path to the image, including a standard image path
		 */
		public String getImagePath() {
			if (pathImage.startsWith("./") || pathImage.startsWith("../")) {
				return pathImage;
			} else {
				return XhtmlSettings.getInstance().local.getDirImages() + pathImage;
			}
		}
		
		/**
		 * Creates the path to the thumbnail.
		 * @return Relative thumbnail path
		 */
		public String getThumbSrc() {
			String thumbSrc = "";
			if (isThumbDesired || isGallery) {
				if (this.thumbSrc.length() > 0){
					return this.thumbSrc;
				} else {
					
					if (pathImage.endsWith(".pdf")) {
						
						/*
						 * Use the pdf image for PDF files, if the user
						 * didn't already give one
						 */
						thumbSrc = Container_Resources.getInstance().rPDFCrystal.getName();
						
						File t = new File(Container_Files.getInstance().cont.targetDir().getAbsolutePath() 
								+ File.separatorChar + XhtmlSettings.getInstance().local.getDirThumbnails()
								+ File.separatorChar + thumbSrc);
						
						if (!t.exists())
							IOUtils.copyFile(Container_Resources.getInstance().rPDFCrystal, t);
						
						if (widthThumb.length() == 0) {
							widthThumb = "100px";
						}
						
					} else if (pathImage.endsWith(".odt") || pathImage.endsWith(".ods") || pathImage.endsWith(".odp") || pathImage.endsWith(".odg")) {
						thumbSrc = Container_Resources.getInstance().rOOo.getName();
						
						File t = new File(Container_Files.getInstance().cont.targetDir().getAbsolutePath() 
								+ File.separatorChar + XhtmlSettings.getInstance().local.getDirThumbnails()
								+ File.separatorChar + thumbSrc);
						
						if (!t.exists())
							IOUtils.copyFile(Container_Resources.getInstance().rOOo, t);
						
						if (widthThumb.length() == 0) {
							widthThumb = "100px";
						}
						
					} else {
						thumbSrc = getThumbnailSource(is.specific.pathImage);
					}
					return thumbSrc;
				}
			} else { // no thumbnail desired
				return pathImage;
			}
		}
		
	}

	
	/**
	 * Replaces: (example: <code>dir/image.jpg</code>)
	 * <ul>
	 * <li>%n &#x2013; full name: <code>dir/image.jpg</code></li>
	 * <li>%f &#x2013; file name: <code>image.jpg</code><li>
	 * <li>%d &#x2013; directory: <code>dir</code></li>
	 * <li>%w &#x2013; without extension: <code>image</code></li>
	 * <li>%e &#x2013; file extension: <code>.jpg</code></li>
	 * </ul>
	 * @param nameThumb The name of the thumbnail
	 * @return The location of a thumbnail
	 */
	public static String getThumbnailSource(String nameThumb) {
		String out;
		if (XhtmlSettings.getInstance().global.dirThumbnails.length() + XhtmlSettings.getInstance().local.dirThumbnails.length() > 0) {
			out = XhtmlSettings.getInstance().local.getDirThumbnails();
			
			// Start replacing
			String n = nameThumb, f = "", d = "", w = "", e = "";
			if (nameThumb.indexOf("/") >= 0) {
				f = nameThumb.substring(nameThumb.lastIndexOf("/") + 1, nameThumb.length());
				d = nameThumb.substring(0, nameThumb.lastIndexOf("/"));
			} else
				f = nameThumb;
			
			if (nameThumb.indexOf(".") >= 0) {
				w = nameThumb.substring(0, nameThumb.lastIndexOf("."));
				e = nameThumb.substring(nameThumb.lastIndexOf("."), nameThumb.length());
			} else
				w = nameThumb;
			out = out.replace("%n", n); 
			out = out.replace("%f", f);
			out = out.replace("%d", d);
			out = out.replace("%w", w);
			out = out.replace("%e", e);
			
		} else
			out = nameThumb;
		
		return out;
	}

}
