package program.settings;

import java.util.Vector;
import java.util.regex.Matcher;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.Constants;
import program.Resources;
import program.Statistics;
import program.WikiLists;
import program.commentator.CommentAtor;
import program.utilities.Sort;
import program.utilities.StringTools;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */

/**
 * 
 * A container for various settings applying header, homelink etc.
 * 
 * @since wiki2xhtml 3.3: Rewritten
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class XhtmlSettings {
	
	public GlobalSettings global = new GlobalSettings();
	
	public LocalSettings local = new LocalSettings();
	
	private static I18n i18n = I18nFactory.getI18n(XhtmlSettings.class, "bin.l10n.Messages", program.Globals.getLocale());
	private static CommentAtor ca = CommentAtor.getInstance();

	private XhtmlSettings() { }
	
	private static XhtmlSettings xhs = new XhtmlSettings();
	
	/** Singleton */
	public static XhtmlSettings getInstance() { return xhs; }
	
	
	/**
	 * Sets the title and replaces %s and %p 
	 * @param title The custom title
	 */
	public void setTitle(String title) {
		if (title == null)
			;
		else {
			int pos = title.indexOf(Constants.Tags.Title.title);
			if (pos >= 0)
				title = title.substring(0, pos)
						+ global.title
						+ title.substring(pos + 2, title.length());
			
			pos = title.indexOf(Constants.Tags.Title.page);
			if (pos >= 0)
				title = title.substring(0, pos)
						+ local.pageAddon
						+ title.substring(pos + 2, title.length());
					
			local.title = title;
			
			ca.ol("setting title: " + title, CommentAtor.DEBUG);
		}
	}

	public void buildMetadata() {
		if (getKeywords().length() > 0)
			local.meta.append(Resources.metaOpen + "keywords" + Resources.metaMiddle
					+ getKeywords() + Resources.metaClose);
		
		if (global.description.length() + local.description.length() > 0)
			local.meta.append(Resources.metaOpen + "description"
					+ Resources.metaMiddle + (local.description.length() > 0 ? local.description : global.description) + Resources.metaClose);
		
		if (global.author.length() + local.author.length() > 0)
			local.meta.append(Resources.metaOpen + "author" + Resources.metaMiddle
					+ (local.author.length() > 0 ? local.author : global.author) + Resources.metaClose);
		
		if (global.lang.length() + local.lang.length() > 0)
			local.meta.append(Resources.metaOpenAlt + "content-language"
					+ Resources.metaMiddle + (local.lang.length() > 0 ? local.lang : global.lang) + Resources.metaClose);
		
		/* IE needs «shortcut icon», standards compliant only take «icon» */
		if (global.icon.length() > 0 || local.icon.length() > 0) 
			local.meta.append("\n  <link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"" + 
					(local.icon.length() > 0? local.icon : global.icon) + "\" />");
		
		if (local.redirect.length() > 0)
			local.meta.append(local.redirect);
		
		if (global.customMeta.length() > 0)
			local.meta.append(global.customMeta);
		if (local.customMeta.length() > 0)
			local.meta.append(local.customMeta);
	}

	/**
	 * Creates a heading entry if there is a heading
	 * @return &lt;h1&gt;heading&lt;/h1&gt;
	 */
	public String mkH1() {
		if (local.h1.length() > 0)
			return "<h1>" + local.h1 + "</h1>";
		return "";
	}

	/**
	 * Creates the link to the start page
	 * @return href="homelink"
	 */
	public String mkHL() {
		if (local.homelink.length() > 0 || global.homelink.length() > 0)
			return " href=\"" + (local.homelink.length() > 0? local.homelink : global.homelink) + "\"";
		else 
			return "";
	}
	
	public void setTOC(StringBuffer tocSource) {
		local.toc = WikiLists.makeList(tocSource);
		local.toc.insert(0, "<div id=\"toc\"><h2>" + Constants.Placeholder.toc + "</h2>");
		local.toc.append("</div><br style=\"clear: left;\" />");
	}
	
	public void setPageAddon(byte number, byte total) {
		local.pageAddon = new StringBuffer(i18n.tr("Page") +' ' + number + ' ' + i18n.tr("of") + ' ' + total);
	}

	public void init() {
		local = new LocalSettings();
	}
	
	/**
	 * @return The used keywords. Keywords set with {{Keyword override common settings. {{AddKeywords doesn't.
	 */
	private String getKeywords() {
		StringBuffer keywords;
		if (local.keywords.length() > 0)
			keywords = local.keywords;
		else 
			keywords = global.keywords;
		
		
		if (local.addedKeywords.length() > 0)
			keywords.append(", " + local.addedKeywords);
		
		/** Statistics timer */
		Statistics.getInstance().sw.timeSortingKeywords.start();
		
		/* Read all keywords into an array */
		Vector<String> v = StringTools.importSimpleCSV(keywords.toString());
		String[] array = new String[v.size()];
		for (int i = 0; i < v.size(); i++)
			array[i] = v.get(i).toLowerCase();
		
		/* Sort the array */
		Sort.shellsort(array);
		
		/* Get the CSV list again */
		v = new Vector<String>();
		String lastItem = "";
		for (int i = 0; i < array.length; i++) {
			if (!array[i].equals(lastItem))
				v.add(array[i]);
			lastItem = array[i];
		}
		keywords = StringTools.exportSimpleCSV(v);
		
		Statistics.getInstance().sw.timeSortingKeywords.stop();
		
		return keywords.toString();
	}
	
	public String getLang() {
		return (local.lang.length() > 0 ? local.lang : global.lang);
	}
	
	/**
	 * Reads the head content, if available.
	 * Head may contain JavaScript, CSS and so on.
	 * @param input Content
	 */
	public void getHead() {
		StringBuffer output = new StringBuffer();
		
		if (local.content != null && local.content.length() > 0) {
			Matcher m = Resources.Regex.head.matcher(local.content);
			
			if (m.find()) {
				output.append(local.content.substring(0, m.start()));
				local.head = m.group(1);
				output.append(local.content.substring(m.end(), local.content.length()));
				local.content = output;
			}
			
		}
	}

	public boolean useImageDescAsCaption() {
		return local.useImageDescAsCaption != null ? local.useImageDescAsCaption : global.useImageDescAsCaption; 
	}
	
	public boolean useImagenameAsCaption() {
		return local.useImagenameAsCaption != null ? local.useImagenameAsCaption : global.useImagenameAsCaption; 
	}
	
	/**
	 * Global settings for all pages
	 */
	public static class GlobalSettings {

		/** Standard author */
		String author = new String();

		/** Standard custom meta data */
		StringBuffer customMeta = new StringBuffer();

		/** Standard description */
		String description = new String();

		/** Standard image page directory; see {@link Constants} */
		public String dirImagepages = Constants.Directories.imagePages;
		
		/** Standard image directory */
		public String dirImages = new String();

		/** Footer */
		public StringBuffer footer = new StringBuffer();

		/** Home link */
		String homelink = new String();

		/** Page icon */
		String icon = new String();

		/** Standard thumbnail source */
		public String dirThumbnails = new String();

		/** Standard keywords */
		StringBuffer keywords = new StringBuffer();

		/** Standard language */
		String lang = new String();
		
		/** Use other reck file */
		public String reckAlternative = "";

		/** Standard text header */
		public String textHeader = new String();
		
		/** Standard page title; replacement for %s */
		String title = new String();
		
		/** Standard width for images on image pages */
		public String widthImgImagepages = Constants.Standards.widthImgImagepages;
		
		/** Standard width for images on image pages */
		public String widthThumbs = Constants.Standards.widthThumbs;
		
		
		/** Standard: Create consistent galleries */
		public boolean useConsistentGallery = Constants.Standards.useConsistentGallery;
		
		/** Standard: Use the image description as caption if no caption available */
		boolean useImageDescAsCaption = Constants.Standards.useImageDescAsCaption;
		
		/** Standard: Don&#x2019;t use image name as caption if no caption available */
		boolean useImagenameAsCaption = Constants.Standards.useImagenameAsCaption;

		
		/** Number of images per line in the gallery */
		public byte galleryImagesPerLine = Constants.Standards.galleryImagesPerLine;
	}
	

	/**
	 * Local settings for a specific page
	 */
	public static class LocalSettings {
		
		/** Setting: This page's keywords */
		StringBuffer addedKeywords = new StringBuffer();
		
		/** Setting: This page's author */
		String author = new String();
		
		/** This page's content */
		public StringBuffer content = new StringBuffer();
		
		/** Setting: This page's custom meta data */
		StringBuffer customMeta = new StringBuffer();
		
		/** Setting: This page's description */
		String description = new String();
		
		/** Setting: Image page directory for current page */
		public String dirImagepages = new String();
		
		/** Setting: Image directory */
		public String dirImages = new String();
		
		/** Setting: This page's thumbnail source */
		public String dirThumbnails = new String();
		
		/** Setting: This page's heading */ 
		String h1 = new String();
		
		/** This page's special header content for JavaScript, CSS and so on */
		private String head = new String();
		
		/** Setting: This page's homelink */
		String homelink = new String();

		/** Setting: This page's icon */
		String icon = new String();
		
		/** Setting: This page's keywords */
		StringBuffer keywords = new StringBuffer();
		
		/** Setting: This page's language */
		String lang = new String();

		/** This page's menu (generated outside) */
		public StringBuffer menu = new StringBuffer();

		/** Setting: This page's meta data */
		public StringBuffer meta = new StringBuffer();
		
		/** This page's number (when multipage) */
		StringBuffer pageAddon = new StringBuffer();
		
		/** Specific file */
		public String reckAlternative = "";
		
		/** Setting: Link to redirected page (given with #REDIRECT [[Link]] */
		String redirect = new String();

		/** Setting: This page's text header */
		public String textHeader = new String();
		
		/** Setting: This page's title */
		public String title = new String();
		
		/** This page's TOC */
		public StringBuffer toc = new StringBuffer();
		
		/** This page's script content, e.g. PHP code */
		public StringBuffer topScript = new StringBuffer();
		
		/** This page's images Standard width for images on image pages */
		public String widthImgImagepages = new String();
		
		/** Standard width for images on image pages */
		public String widthThumbs = new String();
		
		
		/** True, if the current page contains a script (php e.g.) */
		public boolean scriptMode = false;

		/** Overriding global setting */
		public Boolean useConsistentGallery = null;

		/** Overriding global setting */
		Boolean useImageDescAsCaption = null;
		
		/** Overriding global setting */
		Boolean useImagenameAsCaption = null;
		
		
		/** Overriding global setting */
		public Byte galleryImagesPerLine = null;
		

		/** @return The thumbnail directory */
		public String getDirThumbnails() {
			return dirThumbnails.length() > 0 ? dirThumbnails : xhs.global.dirThumbnails; 
		}
		
		/** @return The image directory, with a / at the end */
		public String getDirImages() {
			String dir = dirImages.length() > 0 ? 
					(dirImages.equals(".") ? "" : dirImages) 
					: xhs.global.dirImages;
					
			return (dir.length() > 0 ? dir + "/" : "");
		}
		
		public String getHead(String rule) {
 			if (rule == null || rule.length() == 0 || head.length() == 0) {
				return head;
			}
			
			rule = rule.replace('s', ' ').replace('t', '\t');
			
			return rule + head.replaceAll("\n", "\n" + rule);
		}
		
		public byte getGalleryImagesPerLine() {
			return galleryImagesPerLine != null ? galleryImagesPerLine : XhtmlSettings.getInstance().global.galleryImagesPerLine;
		}
		
		/**
		 * TODO 2 rules for reck files?
		 * @return
		 */
		public String getReck() {
			return (XhtmlSettings.getInstance().local.reckAlternative.length() > 0 ? XhtmlSettings.getInstance().local.reckAlternative : 
				(XhtmlSettings.getInstance().global.reckAlternative.length() > 0 ? XhtmlSettings.getInstance().global.reckAlternative : 
					Constants.Files.reck));
		}
	}

}
