package program.settings;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import program.Constants;
import program.Resources;
import program.Statistics;
import program.commentator.CommentAtor;
import program.utilities.IORead_Stats;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Reads settings from the commons file.
 * 
 * @since wiki2xhtml 3.3 2&#x2a2f; as fast as before (since wiki2xhtml 3.3)
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * 
 */
public class XhtmlSettingsReader {
	
	private static final XhtmlSettings xhs = XhtmlSettings.getInstance();
	
	/** Reads design specific settings */
	public static void getDesignSettings(File f) {
		try {
			
			StringBuffer content = IORead_Stats.readSBuffer(f);
			String value;
			Byte b;
			
			if ((b = getGalleryImagesPerLine(content, false, false)) != null)
				Constants.Standards.galleryImagesPerLine = b;
			if ((value = getWidthImgImagepage(content, false)).length() > 0)
				Constants.Standards.widthImgImagepages = value;
			if ((value = getWidthThumbs(content, false)).length() > 0)
				Constants.Standards.widthThumbs = value;
			if ((value = getGalleryThumbWidth(content, false)).length() > 0)
				Constants.Standards.galleryWidthThumbs = value;
			
			
		} catch (IOException e) {
			CommentAtor.getInstance().ol(
					String.format("I tried to read the file \"%s\", but it was not available for reading.", f.getAbsolutePath()),
					CommentAtor.MSG);
		}
	}
	
	/** Reads the common file. */
	public static void getSettings(StringBuffer content) {
		String temp;
		
		xhs.global.author = getAuthor(content, false);
		xhs.global.description = getDescription(content, false);
		xhs.global.customMeta = getMeta(content, false);
		xhs.global.dirImagepages = ((temp = getDirImagepages(content, false)).length() > 0 ? temp : Constants.Directories.imagePages);
		xhs.global.dirImages = getDirImages(content, false);
		xhs.global.dirThumbnails = getThumbnailDir(content, false);
		xhs.global.galleryImagesPerLine = getGalleryImagesPerLine(content, false, true);
		xhs.global.homelink = getHomelink(content, false);
		xhs.global.icon = getIcon(content, false);
		xhs.global.keywords = getKeywords(content, false);
		xhs.global.lang = getLang(content, false);
		xhs.global.reckAlternative = getReckAlternative(content, false);
		xhs.global.textHeader = getTextHeader(content, false);
		xhs.global.title = getDefaultTitle(content, false);
		xhs.global.widthImgImagepages = ((temp = getWidthImgImagepage(content, false)).length() > 0 ? temp : Constants.Standards.widthImgImagepages);
		xhs.global.widthThumbs = ((temp = getWidthThumbs(content, false)).length() > 0 ? temp : Constants.Standards.widthThumbs);
		
		xhs.global.useImageDescAsCaption = useImageDescAsCaption(content, false, false);
		xhs.global.useImagenameAsCaption = useImagenameAsCaption(content, false, false);
		xhs.global.useConsistentGallery = useConsistentGallery(content, false, false);
	}
	
	/**
	 * Reads local settings. 
	 * @param content
	 * @return
	 */
	public static StringBuffer getXhtml(StringBuffer content) {
		Statistics.getInstance().sw.timeReadingLocalSettings.continueTime();
		
		xhs.setTitle(getTitle(content, true));
		// Use the H1 if no title was specified, but don't delete tag
		if ("".equals(xhs.local.title)) {
			xhs.setTitle(getTitle(content, false));
		}

		xhs.local.author = getAuthor(content, true);
		xhs.local.customMeta = getMeta(content, true);
		xhs.local.description = getDescription(content, true);
		xhs.local.dirImagepages = getDirImagepages(content, true);
		xhs.local.dirImages = getDirImages(content, true);
		xhs.local.dirThumbnails = getThumbnailDir(content, true);
		xhs.local.galleryImagesPerLine = getGalleryImagesPerLine(content, true, false);
		xhs.local.h1 = getH1(content, true);
		xhs.local.lang = getLang(content, true);
		xhs.local.homelink = getHomelink(content, true);
		xhs.local.icon = getIcon(content, true);
		xhs.local.keywords = getKeywords(content, true);
		xhs.local.addedKeywords = getAddKeywords(content, true);
		xhs.local.reckAlternative = getReckAlternative(content, true);
		xhs.local.redirect = getRedirect(content, false);
		xhs.local.textHeader = getTextHeader(content, true);
		xhs.local.widthImgImagepages = getWidthImgImagepage(content, true);
		xhs.local.widthThumbs = getWidthThumbs(content, true);
		
		xhs.local.useImageDescAsCaption = useImageDescAsCaption(content, true, true);
		xhs.local.useImagenameAsCaption = useImagenameAsCaption(content, true, true);
		xhs.local.useConsistentGallery = useConsistentGallery(content, true, true);
		
		Statistics.getInstance().sw.timeReadingLocalSettings.stop();
		
		return content;
	}
	
	private static final String getArg(Pattern pattern, StringBuffer content, boolean remove) {
		Matcher m = pattern.matcher(content);
		String group = "";
		
		if (m.find()) {
			group = m.group(1);
			if (remove) {
				content.delete(m.start(), m.end());
			}
		}
		
		return group;
	}
	
	
	private static final StringBuffer getAddKeywords(StringBuffer content, boolean remove) {
		StringBuffer args = new StringBuffer();
		while (true) {
			String arg = getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.keywordsAdd + ":([^}]+)}}"), content, remove);
			if (arg.length() > 0) {
				args.append(arg);
			} else
				break;
		}
		return args;
	}
	private static final String getAuthor(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.author + ":([^}]+)}}"), content, remove);
	}
	private static final String getDefaultTitle(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.defaultTitle + ":([^}]+)}}"), content, remove);
	}
	private static final String getDescription(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.desc + ":([^}]+)}}"), content, remove);
	}
	private static final String getDirImagepages(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.imagepagesDir + ":([^}]+)\\}\\}"), content, remove);
	}
	private static final Byte getGalleryImagesPerLine(StringBuffer content, boolean remove, boolean useStandard) {
		String arg = getArg(Pattern.compile("\\{\\{" + Constants.Settings.galleryImagesPerLine + ":(\\d+)\\}\\}"), content, remove);
		
		if (arg.length() == 0)
			return useStandard ? Constants.Standards.galleryImagesPerLine : null;
		
		return Byte.parseByte(arg);
	}
	private static final String getH1(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.h1 + ":(.+)}}"), content, remove);
	}
	private static final String getHomelink(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.homelink + ":([^}]+)}}"), content, remove);
	}
	private static final String getIcon(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.icon + ":([^}]+)}}"), content, remove);
	}
	private static final StringBuffer getKeywords(StringBuffer content, boolean remove) {
		return new StringBuffer(getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.keywords + ":([^}]+)}}"), content, remove));
	}
	private static final String getLang(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.lang + ":([^}]+)}}"), content, remove);
	}
	private static final StringBuffer getMeta(StringBuffer content, boolean remove) {
		StringBuffer meta = new StringBuffer();
		
		/* For search engines: Follow all links */
		meta.append("\n  <meta name=\"robots\" content=\"all\" />");
		// or <meta name="robots" content="noindex,nofollow" />
		
		Pattern pMeta = Pattern.compile("(?m)\\{\\{" + Constants.Settings.meta + ":([^}]+)}}");
		Matcher mMeta;
		ArrayList<String> metaList = new ArrayList<String>();
		int pos;
		
		if (remove) {
			while ((mMeta = pMeta.matcher(content)).find()) {
				metaList.add(mMeta.group(1));
				content.replace(mMeta.start(), mMeta.end(), "");
			}
		} else {
			mMeta = pMeta.matcher(content);
			while (mMeta.find()) {
				metaList.add(mMeta.group(1));
			}
		}
		
		for (String s : metaList) {
			if ((pos = s.indexOf("||")) > 0) {
				meta.append("\n  <meta name=\"" + s.substring(0, pos) + "\" content=\"" + s.substring(pos+2, s.length()) + "\" />");
				
			} else {
				meta.append("\n  <meta " + s + "/ >");
			}
		}

		return meta;
	}
	private static final String getTextHeader(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.textHeader + ":([^}]+)}}"), content, remove);
	}
	private static final String getDirImages(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.imagesDir + ":([^}]+)}}"), content, remove);
	}
	private static final String getThumbnailDir(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.thumbsDir + ":([^}]+)}}"), content, remove);
	}
	private static final String getTitle(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.title + Constants.Settings.arg + "}}"), content, remove);
	}
	private static final String getReckAlternative(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("(?m)\\{\\{" + Constants.Settings.reckAlternative + Constants.Settings.arg + "}}"), content, remove);
	}
	private static final String getRedirect(StringBuffer content, boolean remove) {
		Matcher m = Resources.Regex.redirect.matcher(content);
		String redirect = "";
		
		if (m.find()) {
			redirect = "<meta http-equiv=\"refresh\" " +
					"content=\"" + m.group(1) + "; " + "url=" + m.group(2) + " \" />";
			if (remove) {
				content.delete(m.start(), m.end());
			}
		}
		return redirect;
	}
	private static final String getWidthImgImagepage(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("\\{\\{" + Constants.Settings.imagepageImgWidth + ":(\\d+(?:%|px))\\}\\}"), content, remove);
	}
	private static final String getWidthThumbs(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("\\{\\{" + Constants.Settings.thumbWidth + ":(\\d+(?:%|px))\\}\\}"), content, remove);
	}

	private static Boolean useImageDescAsCaption(StringBuffer in, boolean remove, boolean useObject) {
		
		String tag = "{{" + Constants.Settings.descForCaption + "}}";
		int pos;
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return true;
		}
		tag = "{{" + Constants.Settings.descForCaption + Constants.Settings.Args.argFalse + "}}";
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return false;
		}
		return useObject ? null : Constants.Standards.useImageDescAsCaption;
	}
	
	private static String getGalleryThumbWidth(StringBuffer content, boolean remove) {
		return getArg(Pattern.compile("\\{\\{" + Constants.Settings.galleryThumbWidth + ":(\\d+(?:%|px))\\}\\}"), content, remove);
	}
	
	private static Boolean useImagenameAsCaption(StringBuffer in, boolean remove, boolean useObject) {
		
		String tag = "{{" + Constants.Settings.nameForCaption + "}}";
		int pos;
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return true;
		}
		tag = "{{" + Constants.Settings.nameForCaption + Constants.Settings.Args.argFalse + "}}";
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return false;
		}
		return useObject ? null : Constants.Standards.useImagenameAsCaption;
	}
	
	private static Boolean useConsistentGallery(StringBuffer in, boolean remove, boolean useObject) {
		
		String tag = "{{" + Constants.Settings.consistentGallery + "}}";
		int pos;
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return true;
		}
		tag = "{{" + Constants.Settings.consistentGallery + Constants.Settings.Args.argFalse + "}}";
		if ((pos = in.indexOf(tag)) > 0) {
			if (remove) { in.delete(pos, pos + tag.length()); }
			return false;
		}
		
		return useObject ? null : Constants.Standards.useConsistentGallery;
		
	}
	
	

}
