package program.utilities;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.util.regex.*;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 *
 * Various input/output utilities.
 * 
 * @author 	Simon A. Eugster
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * ,		hb9eia
 */
public class IOUtils {
	/**
	 * Replaces the System.out. Now you write o.println(...) instead of
	 * System.out.println(...).
	 * 
	 */
	public static final PrintStream o = System.out;

	public static final PrintStream err = System.err;

	/**
	 * Checks whether a file/directory with this path exists
	 * 
	 * @param filename
	 * @return
	 */
	public static boolean fileExists(String filename) {
		return (new File(filename)).exists();
	}

	/**
	 * Checks whether it is a file (not a directory)
	 * 
	 * @param filename
	 * @return
	 */
	public static boolean isFile(String filename) {
		return (new File(filename)).isFile();
	}

	/**
	 * Checks whether it is a directory (not a file)
	 * 
	 * @param filename
	 * @return
	 */
	public static boolean isDirectory(String filename) {
		return (new File(filename)).isDirectory();
	}

	/**
	 * Creates a new file
	 * 
	 * @param f
	 *            File to create
	 * @return false, if file exists already or an error occured
	 */
	public static boolean newFile(File f) {
		if (f.exists()) {
			return false;
		} else {
			boolean worked = false;
			try {
				f.createNewFile();
				worked = true;
			} catch (IOException e) {
				worked = false;
				System.err.println(e);
			}
			return worked;
		}
	}

	/**
	 * Copies a file
	 * 
	 * @param in -
	 *            input file
	 * @param out -
	 *            output file
	 * @return 0 with no errors, otherwise -1
	 */
	public static boolean copyFile(File in, File out) {
		boolean ok = true;
		InputStream is = null;
		OutputStream os = null;
		try {
			is = new FileInputStream(in);
			os = new FileOutputStream(out);
			byte[] buffer = new byte[0xFFFF];

			for (int len; (len = is.read(buffer)) != -1;)
				os.write(buffer, 0, len);

		} catch (IOException e) {
			System.err.println(e);
			ok = false;
		}

		/* also executed when error occured */
		finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					System.err.println(e);
				}
			}
			if (os != null) {
				try {
					os.close();
				} catch (IOException e) {
					System.err.println(e);
				}
			}
		}
		return ok;
	}

	public static boolean copyDirectory(File in, File out) {
		FileFilter ff = new FileFilter() {

			@Override
			public String getDescription() {
				return null;
			}

			@Override
			public boolean accept(File f) {
				return true;
			}

		};

		return copyDirectory(in, out, ff, false, false);
	}

	public static boolean copyDirectory(File in, File out, FileFilter ff,
			boolean msg, boolean dead) {
		boolean ok = true;

		if (msg && !dead) {
			System.out.println("Copying from " + in.getPath() + " to "
					+ out.getPath());
		}

		if (!isDirectory(in.toString())) {
			ok = false;
			System.err.println("Not a directory! " + in.getPath());
		} else {
			if (out == null) {
				if (!dead) {
					System.err
							.println("Output file is null! (IOUtils.copyDirectory)");
				}
				ok = false;
			} else {
				if (out.exists()) {
					if (out.isFile()) {
						if (!dead)
							System.err
									.println("Warning: "
											+ out.getPath()
											+ " is a file. I can not copy from a directory into a file.");
						ok = false;
					} else {
						if (!dead)
							err.println("Directory already exists: "
									+ out.getPath());
					}
				} else if (!out.mkdirs()) {
					if (!dead) {
						System.err.println("Failed to create directory "
								+ out.getPath());
					}
					ok = false;
				}
			}
		}

		if (ok) {
			File[] fs = in.listFiles();
			String dest;
			for (int i = 0; i < fs.length; i++) {
				if (ff.accept(fs[i])) {
					dest = out.getPath() + File.separatorChar + fs[i].getName();
					if (copyFile(fs[i], new File(dest)))
						;
//					 err.println("copied! " + fs[i].getPath() + " to "
//					 + dest);
				}
			}
		}

		return ok;
	}

	/**
	 * Removes an extension from a file
	 * 
	 * @param fExt -
	 *            File to remove extension from
	 * @return File without an extension (from last point on)
	 */
	public static File removeFileExtension(File fExt) {
		File noExtension = null;
		String filename = fExt.getName();

		int dotPos = filename.lastIndexOf('.'); // get position of last point in
		// Filename
		if (dotPos <= 0) {
			noExtension = fExt; // No '.' found, in fact no extension
		} else {
			noExtension = new File(fExt.getParentFile(), filename.substring(0,
					dotPos));
			// getParentFile: returns file's path
		}

		return noExtension;
	}

	/**
	 * Returns the file extension.
	 * 
	 * @param filename
	 * @return
	 */
	public static String getFileExtensionS(String filename) {
		String ext = new String();

		int dotPos = filename.lastIndexOf('.');
		if (dotPos > 0) {
			ext = filename.substring(dotPos, filename.length());
		}

		return ext;
	}

	/**
	 * Adds an extension to a File.
	 * 
	 * @param file -
	 *            File to add the extension
	 * @param ext -
	 *            Extension, with or without .
	 * @return File with extension added to filename, if necessary
	 */
	public static File addFileExtension(File file, String ext) {
		if (file == null) {
			return null;
		}

		File withExtension = null;

		String filename = file.getName();

		if (filename.endsWith(ext)) {
			return file;
		} else {
			if (ext.startsWith(".") && filename.endsWith("."))
				filename = filename + ext.substring(1, ext.length());
			else { // less than two dots in filename and extension
				if (ext.startsWith(".") || filename.endsWith("."))
					filename = filename + ext;
				else
					// no dot neither in filename nor in extension, so add it
					filename = filename + '.' + ext;
			}
			withExtension = new File(file.getParentFile(), filename);
		}

		return withExtension;
	}

	/**
	 * @param extensions - availible extension
	 * @param directory - the working directory (null for default)
	 * @param title - the title (null for default)
	 * @param append - append selected extension
	 * @return The selected file or null
	 */
	public static File openInFileDialog(String extension, File directory, String title, boolean append) {
		String[] s = new String[1];
		s[0] = extension;
		if (append) {
			return addFileExtension(openInFileDialog(s, directory, title, append), extension);
		} else {
			return openInFileDialog(s, directory, title, append);
		}
	}
	
	/**
	 * @param extensions - availible extensions
	 * @param directory - the working directory (null for default
	 * @param title - the title (null for default)
	 * @param append - append selected extension
	 * @return The selected file or null
	 */
	public static File openInFileDialog(String[] extensions, File directory, String title, boolean append) {
		JFileChooser jfc = new JFileChooser();
		if (title == null) {
			title = new String("Open File");
		}
		jfc.setDialogTitle(title);
		if (directory != null && directory.exists() && directory.isDirectory()) {
			jfc.setCurrentDirectory(directory);
		}
		for (int i = 0; i < extensions.length; i++) {
			final String ext = extensions[i];
			jfc.setFileFilter(new FileFilter() {
				public boolean accept(File f) {
					return f.isDirectory()
							|| f.getName().toLowerCase().endsWith(ext);
				}

				public String getDescription() {
					if (ext.startsWith(".")) {
						return new StringBuffer("*" + ext).toString();
					} else {
						return new StringBuffer("*." + ext).toString();
					}
				}
			});
		}
		int returnValue = jfc.showOpenDialog(null); // showSaveDialog()
		// Opens a file choice dialogue

		File datei;

		if (returnValue == JFileChooser.APPROVE_OPTION) {
			// File selected
			datei = jfc.getSelectedFile();

			String ext = new String();
			String filter = jfc.getFileFilter().getDescription();
			Pattern pat;
			Matcher mat;
			boolean found = false;

			if (append) {
				for (int i = 0; i < extensions.length; i++) {
					pat = Pattern.compile(extensions[i]);
					mat = pat.matcher(filter);
					if (mat.find()) {
						found = true;
						ext = extensions[i];
						break;
					}
				}

				o.println(found);
				if (found) {
					datei = addFileExtension(datei, ext);
				}
			}

			return datei;
		} else {
			return null;
		}
	}

	/**
	 * @author Simon Eugster
	 * @param append -
	 *            append selected extension
	 * @return selected file
	 */
	public static File openInFileDialog(boolean append) {
		String[] s = { ".txt", ".html" };
		return openInFileDialog(s, null, null, append);
	}


	/**
	 * @param extension - availible extension
	 * @param directory - the working directory (null for default
	 * @param title - the title (null for default)
	 * @param append - append selected extension
	 * @return The selected file or null
	 */
	public static File openOutFileDialog(String extension, File directory, String title, boolean append) {
		String[] s = new String[1];
		s[0] = extension;
		if (append) {
			File f = openOutFileDialog(s, directory, title, append);
			if (f != null && !f.exists()) {
				f = addFileExtension(f, extension);
			}
			return f;
		} else {
			return openOutFileDialog(s, directory, title, append);
		}
	}

	/**
	 * @param extensions - availible extensions
	 * @param directory - the working directory (null for default
	 * @param title - the title (null for default)
	 * @param append - append selected extension
	 * @return The selected file or null
	 */
	public static File openOutFileDialog(String[] extensions, File directory, String title, boolean append) {
		JFileChooser jfc = new JFileChooser();
		if (title == null) {
			title = new String("Save File");
		}
		jfc.setDialogTitle("Save File");
		jfc.setDialogType(JFileChooser.SAVE_DIALOG);
		if (directory != null && directory.exists() && directory.isDirectory()) {
			jfc.setCurrentDirectory(directory);
		}
		
		for (int i = 0; i < extensions.length; i++) {
			final String ext = extensions[i];
			jfc.setFileFilter(new FileFilter() {
				public boolean accept(File f) {
					return f.isDirectory()
							|| f.getName().toLowerCase().endsWith(ext);
				}

				public String getDescription() {
					if (ext.startsWith(".")) {
						return new StringBuffer("*" + ext).toString();
					} else {
						return new StringBuffer("*." + ext).toString();
					}
				}
			});
		}

		int returnVal = jfc.showSaveDialog(null); // showSaveDialog()

		File datei;

		if (returnVal == JFileChooser.APPROVE_OPTION) { // File selected
			datei = jfc.getSelectedFile();

			String ext = new String();
			String filter = jfc.getFileFilter().getDescription();
			Pattern pat;
			Matcher mat;
			boolean found = false;

			if (append) {
				for (int i = 0; i < extensions.length; i++) {
					pat = Pattern.compile(extensions[i]);
					mat = pat.matcher(filter);
					if (mat.find()) {
						found = true;
						ext = extensions[i];
						break;
					}
				}

//				o.println(found);
				if (found) {
					datei = addFileExtension(datei, ext);
				}
			}

			return datei;
		} else {
			return null;
		}
	}

	/**
	 * @param append -
	 *            Append selected Extension or not
	 * @return selected file
	 */
	public static File openOutFileDialog(boolean append, File directory) {
		String[] s = { ".html", ".txt" };
		return openOutFileDialog(s, directory, null, append);
	}

	/**
	 * Returns the short path name and eventually removes a part of the path
	 * (usually java.class.dir)
	 * 
	 * @param f
	 * @param remove
	 * @return
	 */
	public static String getShortPath(File f, String remove) {
		StringBuffer shortPath = new StringBuffer();

		try {
			shortPath = new StringBuffer(f.getCanonicalPath());
		} catch (IOException e) {
			System.err.println(e);
		}

		if (remove.length() > 0) {
			if (shortPath.toString().startsWith(remove)) {
				shortPath.delete(0, remove.length());
			}
		}

		return shortPath.toString();
	}

	/**
	 * Checks whether a file is binary by looking for null-Bytes.
	 * 
	 * @param f -
	 *            input file to check
	 * @return true, if it is a binary file
	 */
	public static boolean binaryCheck(File f) {
		boolean bin = false;

//		Stopwatch t = new Stopwatch();
//		t.start();
		try {
			BufferedInputStream bis = new BufferedInputStream(
					new FileInputStream(f));
			byte[] b = new byte[4096];
			int ok = 0;
			while (ok != -1) {
				ok = bis.read(b, 0, b.length);
				for (int i = 0; i < ok; i++) {
					if (b[i] == 0) {
						bin = true;
						ok = -1;
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		// o.println("Checked for binary in " + StringTools.formatTime(t.stop(),
		// true));

		return bin;
	}

	public static void copyAllFiles(File in, File out) {
		try {
			if (in.exists() && in.isDirectory()) {
				if (out.exists() && out.isFile())
					;
				else {
					if (!out.exists()) {
						out.mkdir();
					}
					File[] f = in.listFiles(new FilenameFilter() {
						public boolean accept(File arg0, String arg1) {
							return new File(arg0.getPath() + File.separatorChar
									+ arg1).isFile();
						}
					});
					for (int i = 0; i < f.length; i++) {
						File target = new File(out.getPath()
								+ File.separatorChar + f[i].getName());
						copyFile(f[i], target);
					}
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}